pub enum ProxyPoolStrategy {
RandomPerRequest,
StickyFailover,
}Expand description
Strategy used when selecting a proxy from a ProxyPool.
Variants
RandomPerRequest
Select a random proxy for every request.
StickyFailover
Keep using the same proxy until a failure is observed, then switch to the next proxy.