pub struct Body(/* private fields */);Expand description
An request body.
Implementations
Sourceimpl Body
impl Body
Sourcepub fn as_bytes(&self) -> Option<&[u8]>
pub fn as_bytes(&self) -> Option<&[u8]>
Returns a reference to the internal data of the Body.
None is returned, if the underlying data is a stream.
Sourcepub fn wrap_stream<S>(stream: S) -> Body
Available on crate feature stream only.
pub fn wrap_stream<S>(stream: S) -> Body
stream only.Sourcepub fn into_bytes_mut(self) -> Option<BytesMut>
pub fn into_bytes_mut(self) -> Option<BytesMut>
Convert the body into a mutable bytes::BytesMut if possible.
This is useful for zero-copy operations where you need a mutable buffer, such as with SIMD JSON parsing.
Returns Some(BytesMut) if the body is a reusable Bytes chunk,
None if the body is a streaming body.
Example
use bytes::BytesMut;
use hpx::Body;
let body = Body::from("hello world");
if let Some(bytes_mut) = body.into_bytes_mut() {
// Use mutable bytes for zero-copy operations
println!("Got {} bytes", bytes_mut.len());
}Sourcepub fn reader(self) -> impl AsyncRead
Available on crate feature stream only.
pub fn reader(self) -> impl AsyncRead
stream only.Get an tokio::io::AsyncRead adapter for the body.
This enables efficient streaming of the body using standard async I/O traits.
Example
use hpx::Body;
use tokio::io::AsyncReadExt;
let body = Body::from("hello world");
let mut reader = body.reader();
let mut buffer = Vec::new();
reader.read_to_end(&mut buffer).await?;Optional
This requires the optional stream feature to be enabled.