pub fn post<T: IntoUri>(uri: T) -> RequestBuilderExpand description
Shortcut method to quickly make a POST request.
See also the methods on the hpx::RequestBuilder
type.
NOTE: This function creates a new internal Client on each call,
and so should not be used if making many requests. Create a
Client instead.
Examples
let res = hpx::post("https://httpbin.org/post")
.body("example body")
.send()
.await?;