pub fn put<T: IntoUri>(uri: T) -> RequestBuilderExpand description
Shortcut method to quickly make a PUT request.
See also the methods on the hpx::RequestBuilder
type.
NOTE: This function creates a new internal Client on each call,
and so should not be used if making many requests. Create a
Client instead.
Examples
let res = hpx::put("https://httpbin.org/put")
.body("update content")
.send()
.await?;